<?php
/**
 * NginxHttpPush class file
 *
 * PHP version 5
 *
 * @category JVIBASTA
 * @package  Ext.component
 * @author   Roman Gnidenko <roman444uk@jviba.com>
 * @license  http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link     https://jviba.com/display/PhpDoc/packages
 */
/**
 * NginxHttpPush is a class which push and listen messages channels using web
 * server nginx via nginx_http_push_module 
 * Documentation links: https://pushmodule.slact.net,
 *                      https://pushmodule.slact.net/protocol.html
 *                      http://nginx.org/ru/docs/http/ngx_http_core_module.html
 *
 * @category JVIBASTA
 * @package  Ext.component
 * @author   Roman Gnidenko <roman444uk@jviba.com>
 * @license  http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link     https://jviba.com/display/PhpDoc/packages
 */
class NginxHttpPush extends CApplicationComponent
{
    /**
     * Schema an hostname information
     * @var string information about host
     */
    public $hostInfo;

    /**
     * @var int using connection port (backend-port)
     */
    public $port;
    
    /**
     * @var string pushing path
     */
    public $publishLocation = 'publish';

    /**
     * @var string the url of listening
     */
    public $listenLocation = 'listen';

    /**
     * The name of request parameter which transfers channel ID
     * @var string
     */
    public $channelParamName = 'cid';
    
    /**
     * Default timeout (in seconds) of listening channels
     * @var int
     */
    public $listenTimeout = 100;
    
    /**
     * (non-PHPdoc)
     * @see NginxHttpPush::init()
     */
    public function init()
    {
        $request = Yii::app()->getRequest();
        if (empty($this->frontendHostname)) {
            $this->frontendHostname = $request->getServerName();
        }
        if (empty($this->frontendPort)) {
            $this->frontendPort = $request->getServerPort();
        }
        parent::init();
    }

    /**
     * Push message to the channel
     * 
     * @param string $data          text of the message
     * @param string $channelId     the id of channel
     * @param string $contentType   the content type of the message
     * @param int    &$responceCode pointer to variable to return responce code  
     * 
     * @return boolean true whether message was added to channel, otherwise
     * false (the reason can be that there is no active listeners)
     */
    public function publish($data, $channelId = 'mq', $contentType = 'application/json', &$responceCode = null)
    {
        $client = new EHttpClient($this->hostInfo . '/' . $this->publishLocation);
        if ($this->port) {
            $client->getUri()->setPort($this->port);
        }
        $client->setHeaders('Content-Type', $contentType);
        $client->setParameterGet($this->channelParamName, $channelId);
        $client->setRawData($data);
        $response = $client->request(EHttpClient::POST);
        $responceCode = $response->getStatus();
        return in_array($responceCode, array(200, 201, 202));
    }

    /**
     * Listen the channel and waiting for the message
     * 
     * @param string   $channelId    the ID of channel
     * @param bool     $unfiniteLoop TRUE means wait until server will has respond.
     * @param int|null $timeout      time-out of the single request in seconds. NULL means
     * the default timeout should be used here
     * 
     * @return EHttpResponse null if the channel is closed and instance of EHttpResponse
     * if message is available
     */
    public function listen($channelId, $unfiniteLoop = true, $timeout = null)
    {
        $timeout = $timeout === null ? $this->listenTimeout : $timeout;
        $client = new EHttpClient($this->hostInfo . '/' . $this->listenLocation, array(
            'keepalive' => false,
        ));
        if ($this->port) {
            $client->getUri()->setPort($this->port);
        }
        $client->setParameterGet($this->channelParamName, $channelId);
        $startTime = time();
        $client->setConfig(array(
            'timeout' => $timeout
        ));
        while ($unfiniteLoop || time() - $startTime < $timeout) {
            try {
                $response = $client->request(EHttpClient::GET);
                if ($response instanceof EHttpResponse) {
                    break;
                }
            } catch (EHttpClientException $e) {
                $response = null;
            }
        }
        return $response;
    }
}
